
#include "LiquidCrystal_I2C.h"
#include "Wire.h"
#include "DHT.h"
#define DHTPIN 2     // Digital pin connected to the DHT sensor
#define DHTTYPE DHT11   // DHT 11
#include <Wire.h>
#include <Rtc_Pcf8563.h>
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
//initialize the liquid crystal library
//the first parameter is the I2C address
//the second parameter is how many rows are on your screen
//the third parameter is how many columns are on your screen
LiquidCrystal_I2C lcd(0x27, 16, 2);
DHT dht(DHTPIN, DHTTYPE);
byte mac[] = {0x90, 0xA2, 0xDA, 0x10, 0xA0, 0x55};
unsigned int localPort = 8888;       // local port to listen for UDP packets

const char timeServer[] = "time.nist.gov"; // time.nist.gov NTP server

const int NTP_PACKET_SIZE = 48; // NTP time stamp is in the first 48 bytes of the message

byte packetBuffer[NTP_PACKET_SIZE]; //buffer to hold incoming and outgoing packets

// A UDP instance to let us send and receive packets over UDP
EthernetUDP Udp;
IPAddress ip(192, 168, 0, 121);
EthernetServer server(80);
Rtc_Pcf8563 rtc;
byte Humidity[8] =
{
0b00100,
0b00100,
0b01110,
0b01110,
0b11111,
0b11111,
0b11111,
0b01110
};
byte Temperature[8] =
{
0b01110,
0b01010,
0b01010,
0b01010,
0b10001,
0b10001,
0b10001,
0b01110
};
byte Celsius[8] =
{
0b10000,
0b00111,
0b01000,
0b01000,
0b01000,
0b01000,
0b01000,
0b00111
};

void setup() {
  Serial.begin(9600);
  //initialize lcd screen
  lcd.init();
  // turn on the backlight
  lcd.backlight();
  dht.begin();
  lcd.createChar(0, Humidity);
  lcd.createChar(1, Temperature);
  lcd.createChar(2, Celsius);
  lcd.clear();
  rtc.initClock();
  rtc.setDate(20, 3, 1, 20, 20);
  
  Ethernet.begin(mac);
  Udp.begin(localPort);

  sendNTPpacket(timeServer); // send an NTP packet to a time server

  // wait to see if a reply is available
  delay(1000);
  if (Udp.parsePacket()) {
    // We've received a packet, read the data from it
    Udp.read(packetBuffer, NTP_PACKET_SIZE); // read the packet into the buffer

    // the timestamp starts at byte 40 of the received packet and is four bytes,
    // or two words, long. First, extract the two words:

    unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
    unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);
    // combine the four bytes (two words) into a long integer
    // this is NTP time (seconds since Jan 1 1900):
    unsigned long secsSince1900 = highWord << 16 | lowWord;
    // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
    const unsigned long seventyYears = 2208988800UL;
    // subtract seventy years:
    unsigned long epoch = secsSince1900 - seventyYears;
    rtc.setTime(((epoch  % 86400L) / 3600)+3, (epoch  % 3600) / 60, epoch % 60);
  }
  // wait ten seconds before asking for the time again
  delay(10000);
  Ethernet.maintain();
  Ethernet.begin(mac,ip);
  server.begin( );
}
void sendNTPpacket(const char * address) {
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49;
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); // NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}
void loop() {
  //wait for a second
  delay(1000);
  float h = dht.readHumidity();
  float t = dht.readTemperature();
  if (isnan(h) || isnan(t)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }
  // tell the screen to write on the top row
  lcd.setCursor(0,0);
  lcd.write(byte(0));
  lcd.print(h);
  lcd.print("%");
  lcd.print(" ");
  lcd.print(rtc.formatTime());
  lcd.setCursor(0,1);
  lcd.write(byte(1));
  lcd.print(t);
  lcd.write(byte(2));
  lcd.print(rtc.formatDate());
  EthernetClient client = server.available();
  if (client) 
    {
      boolean currentLineIsBlank = true;
      while (client.connected ( ) ) 
        {
          if (client.available ( ) ) 
            {
              char character = client.read ( );
              Serial.write(character);
              if (character == '\n' && currentLineIsBlank) 
                {
                  client.println ("HTTP/1.1 200 OK");
                  client.println ("Content-Type: text/html");
                  client.println ("Connection: close");
                  client.println ("Refresh: 5");
                  client.println ( );
                  client.println ("<!DOCTYPE HTML>");
                  client.println ("<html>");
                  client.println("<HEAD>");
                  client.println("<meta name='apple-mobile-web-app-capable' content='yes' />");
                  client.println("<meta name='apple-mobile-web-app-status-bar-style' content='black-translucent' />");
                  client.println("<link rel='stylesheet' type='text/css' href='https://unpkg.com/purecss@1.0.1/build/base-min.css' />");
                  client.println("<TITLE>Smart Clock</TITLE>");
                  client.println("</HEAD>");
                  client.print ("<h1>Smart Clock</h1>");
                  client.print ("<h4>Temperature: ");
                  client.print (t);client.print("&deg;C");
                  client.print ("</h4><h4>Humidity: ");
                  client.print (h);client.print("%");
                  client.println ("<br />");
                  client.println ("</html>");
                  break;
                }
                 
                if ( character == '\n') 
                  {
                    currentLineIsBlank = true;
                  } 
                else if (character != '\r') 
                  {
                    currentLineIsBlank = false;
            }
        }
    }
    delay(1);
    client.stop();
  }
  
}
